from petisco.domain.value_objects.user_id import UserId

DEFAULT_ORGANIZATION = "alice"
DEFAULT_SERVICE = "petisco"
DEFAULT_EXCHANGE_NAME = f"{DEFAULT_ORGANIZATION}.{DEFAULT_SERVICE}"
DEFAULT_USER_ID = UserId("userpwlxU3yGsird")
DEFAULT_MAX_RETRIES = 5
DEFAULT_VERBOSE = True
